with Pins;
with Utils;

package body Program is
   
   type States is (Off, On);
   
   procedure Blink (State : in out States; P : in Pins.Pin_ID) is
   begin
      if State = Off then
         State := On;
      else
         State := Off;
      end if;
      
      Pins.Write (P, State = On);
   end Blink;

   State_10 : States;
   State_11 : States;
   State_12 : States;
   
   procedure Blink_10 is
   begin
      Blink (State_10, Pins.D10);
   end Blink_10;
   
   procedure Blink_11 is
   begin
      Blink (State_11, Pins.D11);
   end Blink_11;
   
   procedure Blink_12 is
   begin
      Blink (State_12, Pins.D12);
   end Blink_12;
   
   Tasks : Utils.Task_Descriptions (1 .. 3);
   
   procedure Run is
      Next_Time : Utils.Time;

      use type Utils.Time;
   begin
      Pins.Enable_Output (Pins.D10);
      Pins.Enable_Output (Pins.D11);
      Pins.Enable_Output (Pins.D12);
      
      Utils.Enable_Interrupts;
      Utils.Enable_System_Time;
      
      State_10 := Off;
      State_11 := Off;
      State_12 := Off;
      
      Tasks :=
        (1 => (Action => Blink_10'Access, Activation_Period => 1000, Time_Since_Last_Due_Active => 0),
         2 => (Action => Blink_11'Access, Activation_Period => 700, Time_Since_Last_Due_Active => 0),
         3 => (Action => Blink_12'Access, Activation_Period => 700, Time_Since_Last_Due_Active => 350));

      Next_Time := Utils.Clock;
      loop
         Utils.Scheduler_Tick (Tasks, Utils.Base_Tick);
         
         Next_Time := Next_Time + Utils.Base_Tick;
         Utils.Delay_Until (Next_Time);
      end loop;
   end Run;

end Program;
